@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright: 2021, Claudio S. Ravasio
+# Copyright: 2022, Claudio S. Ravasio
 # License: MIT (https://opensource.org/licenses/MIT)
 # Author: Claudio S. Ravasio, PhD student at University College London (UCL), research assistant at King's College
 # London (KCL), supervised by:
@@ -14,4 +14,5 @@
 from .flow_class import Flow
 from .flow_operations import *
 from .utils import from_matrix, from_transforms, load_kitti, load_sintel, load_sintel_mask, \
-    resize_flow, apply_flow, is_zero_flow, track_pts
+    resize_flow, apply_flow, is_zero_flow, track_pts, get_pure_pytorch, set_pure_pytorch, unset_pure_pytorch, \
+    to_numpy, to_tensor, move_axis, show_masked_image, apply_s_flow, grid_from_unstructured_data
