@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright: 2021, Claudio S. Ravasio
+# Copyright: 2022, Claudio S. Ravasio
 # License: MIT (https://opensource.org/licenses/MIT)
 # Author: Claudio S. Ravasio, PhD student at University College London (UCL), research assistant at King's College
 # London (KCL), supervised by:
@@ -16,7 +16,7 @@ import numpy as np
 from typing import Union
 import torch
 from .flow_class import Flow
-from .utils import validate_shape
+from .utils import get_valid_shape
 
 
 FlowAlias = 'Flow'
@@ -44,7 +44,7 @@ def visualise_definition(
 
     # Default arguments and input validation
     shape = [601, 601] if shape is None else shape
-    validate_shape(shape)
+    shape = get_valid_shape(shape)[1:]
     insert_text = True if insert_text is None else insert_text
     if not isinstance(insert_text, bool):
         raise TypeError("Error visualising the flow definition: Insert_text needs to be a boolean")
@@ -57,7 +57,7 @@ def visualise_definition(
     flow = Flow.from_transforms([['scaling', w//2, h//2, 1.1]], shape)
 
     flow.vecs = (torch.abs(flow.vecs) ** 1.2) * torch.sign(flow.vecs)
-    img = flow.visualise(mode, return_tensor=False).astype('f')  # dtype 'f' necessary for cv2.arrowedLine
+    img = flow.visualise(mode, return_tensor=False).astype('f')[0]  # dtype 'f' necessary for cv2.arrowedLine
 
     # Draw on the flow image
     line_colour = (0, 0, 0)
@@ -90,12 +90,19 @@ def combine_flows(
 ) -> Union[FlowAlias, torch.Tensor]:
     """Returns the result of the combination of two flow fields of the same shape and reference
 
+    If the toolbox-wide variable ``PURE_PYTORCH`` is set to ``True`` (default, see also
+    :meth:`~oflibpytorch.set_pure_pytorch`), the output flow field tensor is differentiable with respect to the
+    input flow fields, if given as tensors.
+
     .. tip::
        All of the flow field combinations in this function rely on some combination of the
-       :meth:`~oflibnumpy.Flow.apply`, :meth:`~oflibnumpy.Flow.invert`, and :func:`~oflibnumpy.Flow.combine_with`
-       methods, and can be very slow (several seconds) due to calling :func:`scipy.interpolate.griddata` multiple
-       times. The table below aids decision-making with regards to which reference a flow field should be provided
-       in to obtain the fastest result.
+       :meth:`~oflibpytorch.Flow.apply`, :meth:`~oflibpytorch.Flow.invert`, and :func:`~oflibpytorch.Flow.combine_with`
+       methods.
+
+       If ``PURE_PYTORCH`` is set to ``False``, some of these methods will call :func:`scipy.interpolate.griddata`,
+       possibly multiple times, which can be very slow (several seconds) - but the result will be more accurate
+       compared to using the PyTorch-only setting. The table below aids decision-making with regards to which
+       reference a flow field should be provided in to obtain the fastest result.
 
         .. list-table:: Calls to :func:`scipy.interpolate.griddata`
            :header-rows: 1
@@ -147,8 +154,12 @@ def combine_flows(
        by a rotation is not the same as a rotation followed by a translation: adding up vectors at each pixel cannot be
        the correct solution as there wouldn't be a difference based on the order of vector addition.
 
-    :param input_1: First input flow as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`.
-        Can also be a flow object, but this will be deprecated soon
+    :param input_1: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as
+        :math:`(2, H, W)` or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or
+        :math:`(N, H, W, 2)`, throwing a ``ValueError`` if this isn't possible either. The dimension that is 2
+        (the channel dimension) contains the flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the
+        horizontal, ``flow_vectors[..., 1]`` are the vertical vector components, defined as positive when pointing
+        to the right / down. Can also be a flow object, but this will be deprecated soon
     :param input_2: Second input flow, same type as ``input_1``
     :param mode: Integer determining how the input flows are combined, where the number corresponds to the position in
         the formula :math:`flow_1 ⊕ flow_2 = flow_3`:
@@ -161,8 +172,9 @@ def combine_flows(
           be :math:`flow_3`
     :param ref: The reference of the input flow fields, either ``s`` or ``t``
     :param thresholded: Boolean determining whether flows are thresholded during an internal call to
-        :meth:`~oflibnumpy.Flow.is_zero`, defaults to ``False``
-    :return: Flow object if inputs are flow objects (deprecated in future, avoid), Torch tensor as standard
+        :meth:`~oflibpytorch.Flow.is_zero`, defaults to ``False``
+    :return: Flow object if inputs are flow objects (deprecated in future, avoid), Torch tensor of shape
+        :math:`(2, H, W)` or :math:`(N, 2, H, W)` as standard
     """
 
     if isinstance(input_1, Flow) and isinstance(input_2, Flow):
@@ -173,35 +185,47 @@ def combine_flows(
         return result
 
     result = Flow(input_1, ref).combine_with(Flow(input_2, ref), mode=mode, thresholded=thresholded)
-    return result.vecs
+    return result.vecs if len(input_1.shape) > 3 else result.vecs.squeeze(0)
 
 
 def switch_flow_ref(flow: Union[np.ndarray, torch.Tensor], input_ref: str) -> torch.Tensor:
     """Recalculate flow vectors to correspond to a switched flow reference (see Flow reference
-    :attr:`~oflibnumpy.Flow.ref`)
+    :attr:`~oflibpytorch.Flow.ref`)
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    The output flow field tensor is differentiable with respect to the input flow field, if given as a tensor.
+
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param input_ref: The reference of the input flow field, either ``s`` or ``t``
-    :return: Flow field as a torch tensor of shape :math:`(2, H, W)`
+    :return: Flow field as a torch tensor of shape :math:`(2, H, W)` or :math:`(N, 2, H, W)`
     """
 
     f = Flow(flow, input_ref).switch_ref()
-    return f.vecs
+    return f.vecs if len(flow.shape) > 3 else f.vecs.squeeze(0)
 
 
 def invert_flow(flow: Union[np.ndarray, torch.Tensor], input_ref: str, output_ref: str = None) -> torch.Tensor:
     """Inverting a flow: `img`\\ :sub:`1` -- `f` --> `img`\\ :sub:`2` becomes `img`\\ :sub:`1` <-- `f` --
     `img`\\ :sub:`2`. The smaller the input flow, the closer the inverse is to simply multiplying the flow by -1.
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    The output flow field tensor is differentiable with respect to the input flow field, if given as a tensor.
+
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param input_ref: Reference of the input flow field,  either ``s`` or ``t``
     :param output_ref: Desired reference of the output field, either ``s`` or ``t``. Defaults to ``input_ref``
-    :return: Flow field as a torch tensor of shape :math:`(2, H, W)`
+    :return: Flow field as a torch tensor of shape :math:`(2, H, W)` or :math:`(N, 2, H, W)`
     """
 
     output_ref = input_ref if output_ref is None else output_ref
     f = Flow(flow, input_ref).invert(output_ref)
-    return f.vecs
+    return f.vecs if len(flow.shape) > 3 else f.vecs.squeeze(0)
 
 
 def valid_target(flow: Union[np.ndarray, torch.Tensor], ref: str) -> torch.Tensor:
@@ -217,17 +241,22 @@ def valid_target(flow: Union[np.ndarray, torch.Tensor], ref: str) -> torch.Tenso
     actual information being available at this position (validity ``False``), and pixels that are black due to black
     pixel values having been warped to that (valid) location by the flow.
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
-    :return: Boolean torch tensor of the same shape :math:`(H, W)` as the flow
+    :return: Boolean torch tensor of the same shape :math:`(H, W)` or :math:`(N, H, W)` as the flow
     """
 
-    return Flow(flow, ref).valid_target()
+    t = Flow(flow, ref).valid_target()
+    return t if len(flow.shape) > 3 else t.squeeze(0)
 
 
 def valid_source(flow: Union[np.ndarray, torch.Tensor], ref: str) -> torch.Tensor:
     """Finds the area in the source domain that will end up being valid in the target domain (see
-    :meth:`~oflibnumpy.valid_target`) after warping
+    :meth:`~oflibpytorch.valid_target`) after warping
 
     Given a source image and a flow, both of shape :math:`(H, W)`, the target image is created by warping the source
     with the flow. The source area is then a boolean numpy array of shape :math:`(H, W)` that is ``True`` wherever
@@ -235,12 +264,17 @@ def valid_source(flow: Union[np.ndarray, torch.Tensor], ref: str) -> torch.Tenso
     source will either be warped outside of the target image, or not be warped at all due to a lack of valid flow
     vectors connecting to this position.
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
-    :return: Boolean torch tensor of the same shape :math:`(H, W)` as the flow
+    :return: Boolean torch tensor of the same shape :math:`(H, W)` or :math:`(N, H, W)` as the flow
     """
 
-    return Flow(flow, ref).valid_source()
+    s = Flow(flow, ref).valid_source()
+    return s if len(flow.shape) > 3 else s.squeeze(0)
 
 
 def get_flow_padding(flow: Union[np.ndarray, torch.Tensor], ref: str) -> list:
@@ -249,18 +283,24 @@ def get_flow_padding(flow: Union[np.ndarray, torch.Tensor], ref: str) -> list:
     - When the flow reference is ``t`` ("target"), this corresponds to the padding needed in a source image which
       ensures that every flow vector will find a value in the source domain to warp towards the target domain.
       I.e. any invalid locations in the area :math:`H \\times W` of the target domain (see
-      :func:`~oflibnumpy.valid_target`) are purely due to no valid flow vector being available to pull a
+      :func:`~oflibpytorch.valid_target`) are purely due to no valid flow vector being available to pull a
       source value to this target location, rather than no source value being available in the first place.
     - When the flow reference is ``s`` ("source"), this corresponds to the padding needed for
       the flow itself, so that applying it to a source image will result in no input image information being lost in
       the warped output, i.e each input image pixel will come to lie inside the padded area.
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
-    :return: A list of shape :math:`(4)` with the values ``[top, bottom, left, right]``
+    :return: A list of shape :math:`(4)` or :math:`(N, 4)` with the values ``[top, bottom, left, right]`` for each
+        batch member, if applicable
     """
 
-    return Flow(flow, ref).get_padding()
+    p = Flow(flow, ref).get_padding()
+    return p if len(flow.shape) > 3 else p[0]
 
 
 def get_flow_matrix(
@@ -271,7 +311,11 @@ def get_flow_matrix(
 ) -> torch.Tensor:
     """Fit a transformation matrix to the flow field using OpenCV functions
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
     :param dof: Integer describing the degrees of freedom in the transformation matrix to be fitted, defaults to
         ``8``. Options are:
@@ -285,10 +329,11 @@ def get_flow_matrix(
         - ``lms``: Least mean squares
         - ``ransac``: RANSAC-based robust method
         - ``lmeds``: Least-Median robust method
-    :return: Torch tensor of shape :math:`(3, 3)` containing the transformation matrix
+    :return: Torch tensor of shape :math:`(3, 3)` or :math:`(N, 3, 3)` containing the transformation matrix
     """
 
-    return Flow(flow, ref).matrix(dof=dof, method=method)
+    m = Flow(flow, ref).matrix(dof=dof, method=method)
+    return m if len(flow.shape) > 3 else m.squeeze(0)
 
 
 def visualise_flow(
@@ -299,17 +344,27 @@ def visualise_flow(
 ) -> Union[np.ndarray, torch.Tensor]:
     """Visualises the flow as an rgb / bgr / hsv image
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    .. note::
+        This currently runs internally based on NumPy & OpenCV, due to a lack of easily accessible equivalent
+        functions for coordinate and colour space conversions. Therefore, even if the output is a tensor, it
+        will not be differentiable with respect to the input flow tensor.
+
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param mode: Output mode, options: ``rgb``, ``bgr``, ``hsv``
     :param range_max: Maximum vector magnitude expected, corresponding to the HSV maximum Value of 255 when scaling
         the flow magnitudes. Defaults to the 99th percentile of the flow field magnitudes
     :param return_tensor: Boolean determining whether the result is returned as a tensor. Note that the result is
         originally a numpy array. Defaults to ``True``
-    :return: Numpy array of shape :math:`(H, W, 3)` or torch tensor of shape :math:`(3, H, W)` containing the
-        flow visualisation
+    :return: Numpy array of shape :math:`(H, W, 3)` or :math:`(N, H, W, 3)` or torch tensor of shape :math:`(3, H, W)`
+        or :math:`(N, 3, H, W)` containing the flow visualisation
     """
 
-    return Flow(flow).visualise(mode=mode, range_max=range_max, return_tensor=return_tensor)
+    v = Flow(flow).visualise(mode=mode, range_max=range_max, return_tensor=return_tensor)
+    return v if len(flow.shape) > 3 else v.squeeze(0)
 
 
 def visualise_flow_arrows(
@@ -324,7 +379,16 @@ def visualise_flow_arrows(
 ) -> Union[np.ndarray, torch.Tensor]:
     """Visualises the flow as arrowed lines
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    .. note::
+        This currently runs internally based on NumPy & OpenCV, due to a lack of easily accessible equivalent
+        functions for coordinate and colour space conversions. Therefore, even if the output is a tensor, it
+        will not be differentiable with respect to the input flow tensor.
+
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        or :math:`(N, 2, H, W)` if possible, otherwise as :math:`(H, W, 2)` or :math:`(N, H, W, 2)`, throwing a
+        ``ValueError`` if this isn't possible either. The dimension that is 2 (the channel dimension) contains the
+        flow vector in OpenCV convention: ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are
+        the vertical vector components, defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
     :param grid_dist: Integer of the distance of the flow points to be used for the visualisation, defaults to
         ``20``
@@ -332,22 +396,27 @@ def visualise_flow_arrows(
     :param scaling: Float or int of the flow line scaling, defaults to scaling the 99th percentile of arrowed line
         lengths to be equal to twice the grid distance (empirical value)
     :param colour: Tuple of the flow arrow colour, defaults to hue based on flow direction as in
-        :func:`~oflibnumpy.visualise`
+        :func:`~oflibpytorch.visualise`
     :param thickness: Integer of the flow arrow thickness, larger than zero. Defaults to ``1``
     :param return_tensor: Boolean determining whether the result is returned as a tensor. Note that the result is
         originally a numpy array. Defaults to ``True``
-    :return: Numpy array of shape :math:`(H, W, 3)` or torch tensor of shape :math:`(3, H, W)` containing the
-        flow visualisation
+    :return: Numpy array of shape :math:`(H, W, 3)` or :math:`(N, H, W, 3)` or torch tensor of shape :math:`(3, H, W)`
+        or :math:`(N, 3, H, W)` containing the flow visualisation
     """
 
-    return Flow(flow, ref).visualise_arrows(grid_dist=grid_dist, img=img, scaling=scaling,
-                                            colour=colour, thickness=thickness, return_tensor=return_tensor)
+    a = Flow(flow, ref).visualise_arrows(grid_dist=grid_dist, img=img, scaling=scaling,
+                                         colour=colour, thickness=thickness, return_tensor=return_tensor)
+    return a if len(flow.shape) > 3 else a.squeeze(0)
 
 
 def show_flow(flow: Union[np.ndarray, torch.Tensor], wait: int = None):
-    """Shows the flow in an OpenCV window using :func:`~oflibnumpy.visualise`
+    """Shows the flow in an OpenCV window using :func:`~oflibpytorch.visualise`
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        if possible, otherwise as :math:`(H, W, 2)`, throwing a ``ValueError`` if this isn't possible either. The
+        dimension that is 2 (the channel dimension) contains the flow vector in OpenCV convention:
+        ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are the vertical vector components,
+        defined as positive when pointing to the right / down.
     :param wait: Integer determining how long to show the flow for, in milliseconds. Defaults to ``0``, which means
         it will be shown until the window is closed, or the process is terminated
     """
@@ -364,9 +433,13 @@ def show_flow_arrows(
     scaling: Union[float, int] = None,
     colour: tuple = None
 ):
-    """Shows the flow in an OpenCV window using :func:`~oflibnumpy.visualise_arrows`
+    """Shows the flow in an OpenCV window using :func:`~oflibpytorch.visualise_arrows`
 
-    :param flow: Flow field as a numpy array or torch tensor, shape :math:`(2, H, W)` or :math:`(H, W, 2)`
+    :param flow: Numpy array or pytorch tensor with 3 or 4 dimension. The shape is interpreted as :math:`(2, H, W)`
+        if possible, otherwise as :math:`(H, W, 2)`, throwing a ``ValueError`` if this isn't possible either. The
+        dimension that is 2 (the channel dimension) contains the flow vector in OpenCV convention:
+        ``flow_vectors[..., 0]`` are the horizontal, ``flow_vectors[..., 1]`` are the vertical vector components,
+        defined as positive when pointing to the right / down.
     :param ref: Reference of the flow field, ``s`` or ``t``
     :param wait: Integer determining how long to show the flow for, in milliseconds. Defaults to ``0``, which means
         it will be shown until the window is closed, or the process is terminated
@@ -376,7 +449,35 @@ def show_flow_arrows(
     :param scaling: Float or int of the flow line scaling, defaults to scaling the 99th percentile of arrowed line
         lengths to be equal to twice the grid distance (empirical value)
     :param colour: Tuple of the flow arrow colour, defaults to hue based on flow direction as in
-        :func:`~oflibnumpy.visualise`
+        :func:`~oflibpytorch.visualise`
     """
 
     return Flow(flow, ref).show_arrows(wait=wait, grid_dist=grid_dist, img=img, scaling=scaling, colour=colour)
+
+
+def batch_flows(flows: Union[list, tuple]) -> FlowAlias:
+    """Returns a batched flow object from a list of input flows of the same size and flow reference :attr:`ref`.
+
+    The output flow vectors are differentiable with respect to the input flow vectors.
+
+    :param flows: Tuple or list of flow objects. Flow objects to have the same flow reference
+        :attr:`~oflibpytorch.Flow.ref`, as well as the same flow field heights and widths :math:`(H, W)`. They can have
+        any batch size.
+    :return: Single batched flow object, with a batch size equal to the sum of all individual input batch sizes
+    """
+
+    if not isinstance(flows, (list, tuple)):
+        raise TypeError("Error batching flows: Input flow objects need to be passed in a list or tuple")
+    if any(not isinstance(f, Flow) for f in flows):
+        raise TypeError("Error batching flows: All inputs need to be flow objects")
+    if any(f.ref != flows[0].ref for f in flows):
+        raise ValueError("Error batching flows: All input flow objects need to have the same reference")
+    if any(f.device != flows[0].device for f in flows):
+        raise ValueError("Error batching flows: All input flow objects need to have the same device")
+    if any(f.shape[1:] != flows[0].shape[1:] for f in flows):
+        raise ValueError("Error batching flows: All input flow objects need to have the same height and width")
+
+    batched_vecs = torch.cat(tuple(f.vecs for f in flows), dim=0)
+    batched_masks = torch.cat(tuple(f.mask for f in flows), dim=0)
+
+    return Flow(batched_vecs, flows[0].ref, batched_masks, flows[0].device)
